#include "../../std.h"

#include <string>
#ifdef HAVE_SSTREAM
#include <sstream>
#else
#include <strstream>
#endif

#include "../../misc/log_time.h"
#include "../../misc/escaped.h"
#include "../../misc/time_utils.h"

#include "beswsi.h"
#include "beswsi_soapH.h"

extern SOAP_NMAC struct Namespace beswsi_soap_namespaces[];

class HTTP_BESWSI_Handle {
 friend class HTTP_BESWSI;
 private:
  plugin_init_t jobplugin_init;
  void* jobplugin_handle;
 public:
  HTTP_BESWSI_Handle(const char* path);
  ~HTTP_BESWSI_Handle(void);
};

HTTP_BESWSI_Handle::HTTP_BESWSI_Handle(const char* path) {
  jobplugin_handle=dlopen(path,RTLD_NOW);
  if(!jobplugin_handle) {
    odlog(ERROR)<<dlerror()<<std::endl;
    odlog(ERROR)<<"Can't load plugin "<<path<<std::endl;
    return;
  };
  jobplugin_init=(plugin_init_t)dlsym(jobplugin_handle,"init");
  if(jobplugin_init == NULL) {
    olog<<"Plugin "<<path<<" is broken."<<std::endl;
    dlclose(jobplugin_handle); jobplugin_handle=NULL; return;
  };
}

HTTP_Service* beswsi_service_creator(HTTP_Connector& c,const char* uri,void* arg) {
  HTTP_BESWSI_Handle& handle = *((HTTP_BESWSI_Handle*)arg);
  HTTP_BESWSI *h = new HTTP_BESWSI(&c,&handle);
  return h;
}

bool beswsi_service_configurator(std::istream& f,const char* uri,HTTP_Service_Properties &prop) {
  std::string jobplugin_path;
  for(;!f.eof();) {
    char buf[1024];
    istream_readline(f,buf,sizeof(buf));
    char* p = buf;
    for(;*p;p++) if(!isspace(*p)) break;
    char* command = p;
    if(*command == '#') continue;
    for(;*p;p++) if(isspace(*p)) break;
    int command_len = p-command;
    if(!command_len) continue;
    if((command_len==3) && (strncmp(command,"end",command_len)==0)) { break; }
    else if((command_len==9) && (strncmp(command,"jobplugin",command_len)==0)) {      input_escaped_string(p,jobplugin_path);
    };
  };
  HTTP_BESWSI_Handle* handle = new HTTP_BESWSI_Handle(jobplugin_path.c_str());
  prop.arg=handle;
  prop.subtree=false;
  return true;
}

HTTP_BESWSI::HTTP_BESWSI(HTTP_Connector *c,HTTP_BESWSI_Handle* h):HTTP_ServiceAdv(c) {
  /* SOAP */
  soap_init();
  sp.namespaces=beswsi_soap_namespaces;
  sp.user=this;
  // Initialize plugin's instance
  userspec.fill(c->identity());
  std::string jobplugin_config = "end\n";
#ifdef HAVE_SSTREAM
  std::stringstream cfile(jobplugin_config);
#else
  std::strstream cfile;
  cfile<<jobplugin_config;
#endif
  if(h->jobplugin_init) {
    nordugrid_config_loc="";
    unsetenv("NORDUGRID_CONFIG");
    unsetenv("ARC_CONFIG");
    jobplugin=(*(h->jobplugin_init))(cfile,userspec);
    if(jobplugin == NULL) {
      olog<<"Job control plugin failed to initialise." << std::endl;
      return;
    };
  };
}

HTTP_BESWSI::~HTTP_BESWSI(void) {
}

// HTTP methods

HTTP_Error HTTP_BESWSI::get(const char* uri,int &keep_alive) {
  return HTTP_NOT_IMPLEMENTED;
}

HTTP_Error HTTP_BESWSI::put(const char* uri,int &keep_alive) {
  return HTTP_NOT_IMPLEMENTED;
}

HTTP_Error HTTP_BESWSI::post(const char* uri,int &keep_alive) {
  //const char* name = requested_path.c_str();
  //for(;*name;++name) if(*name != '/') break;
  //int l = strlen(name);
  //for(;l;--l) { if(name[l-1] != '/') break; };
  //job_id.assign(name,l); job_only=true;
  //if(job_id.find('/') != std::string::npos) job_only=false;
  //if(!job_only) return HTTP_NOT_FOUND;
  HTTP_Error r = soap_post(uri,keep_alive);
  return r;
}

// gSOAP switch

#ifdef OLD_GSOAP
#error gSOAP older than 2.5.2 is not supported.
#else
void HTTP_BESWSI::soap_methods(void) {
  odlog(VERBOSE)<<"soap_methods: tag: "<<sp.tag<<std::endl;
  if(!soap_match_tag(&sp,sp.tag,"beswsi:CreateActivityFromJSDL")) soap_serve___beswsi__CreateActivityFromJSDL(&sp) ;
  else if(!soap_match_tag(&sp,sp.tag,"beswsi:GetActivityStatus")) soap_serve___beswsi__GetActivityStatus(&sp);
  else if(!soap_match_tag(&sp,sp.tag,"beswsi:RequestActivityStateChanges")) soap_serve___beswsi__RequestActivityStateChanges(&sp);
  else if(!soap_match_tag(&sp,sp.tag,"beswsi:StopAcceptingNewActivities")) soap_serve___beswsi__StopAcceptingNewActivities(&sp);
  else if(!soap_match_tag(&sp,sp.tag,"beswsi:StartAcceptingNewActivities")) soap_serve___beswsi__StartAcceptingNewActivities(&sp);
  else if(!soap_match_tag(&sp,sp.tag,"beswsi:IsAcceptingNewActivities")) soap_serve___beswsi__IsAcceptingNewActivities(&sp);
  else if(!soap_match_tag(&sp,sp.tag,"beswsi:GetActivityJSDLDocuments")) soap_serve___beswsi__GetActivityJSDLDocuments(&sp);
  else {
    sp.error=SOAP_NO_METHOD;
    soap_flush(&sp);
  };
}
#endif

// SOAP methods

int  __beswsi__CreateActivityFromJSDL(struct soap* sp, _bes__CreateActivityFromJSDL *bes__CreateActivityFromJSDL, _bes__CreateActivityFromJSDLResponse *bes__CreateActivityFromJSDLResponse) {


    jsdl__JobDefinition_USCOREType *jsdl__JobDefinition;
        class jsdl__JobDescription_USCOREType *JobDescription;


    bool create_suspended;  /* required element of type xsd:boolean */




  HTTP_BESWSI& it = *((HTTP_BESWSI*)(sp->user));
  return SOAP_NO_METHOD;
}

int  __beswsi__GetActivityStatus(struct soap* sp, _bes__GetActivityStatus *bes__GetActivityStatus, _bes__GetActivityStatusResponse *bes__GetActivityStatusResponse) {
  HTTP_BESWSI& it = *((HTTP_BESWSI*)(sp->user));
  return SOAP_NO_METHOD;
}

int  __beswsi__RequestActivityStateChanges(struct soap* sp, _bes__RequestActivityStateChanges *bes__RequestActivityStateChanges, _bes__RequestActivityStateChangesResponse *bes__RequestActivityStateChangesResponse) {
  HTTP_BESWSI& it = *((HTTP_BESWSI*)(sp->user));
  return SOAP_NO_METHOD;
}

int  __beswsi__StopAcceptingNewActivities(struct soap* sp, _bes__StopAcceptingNewActivities bes__StopAcceptingNewActivities, _bes__StopAcceptingNewActivitiesResponse &bes__StopAcceptingNewActivitiesResponse) {
  HTTP_BESWSI& it = *((HTTP_BESWSI*)(sp->user));
  return SOAP_NO_METHOD;
}

int  __beswsi__StartAcceptingNewActivities(struct soap* sp, _bes__StartAcceptingNewActivities bes__StartAcceptingNewActivities, _bes__StartAcceptingNewActivitiesResponse &bes__StartAcceptingNewActivitiesResponse) {
  HTTP_BESWSI& it = *((HTTP_BESWSI*)(sp->user));
  return SOAP_NO_METHOD;
}

int  __beswsi__IsAcceptingNewActivities(struct soap* sp, _bes__IsAcceptingNewActivities bes__IsAcceptingNewActivities, _bes__IsAcceptingNewActivitiesResponse *bes__IsAcceptingNewActivitiesResponse) {
  HTTP_BESWSI& it = *((HTTP_BESWSI*)(sp->user));
  return SOAP_NO_METHOD;
}

int __beswsi__GetActivityJSDLDocuments(struct soap* sp, _bes__GetActivityJSDLDocuments *bes__GetActivityJSDLDocuments, _bes__GetActivityJSDLDocumentsResponse *bes__GetActivityJSDLDocumentsResponse) {
  HTTP_BESWSI& it = *((HTTP_BESWSI*)(sp->user));
  return SOAP_NO_METHOD;
}

