/*
 * Created on Jul 26, 2005
 *
 * TODO Implement it, then write some documentation
 *
 */
package se.umu.jobsubmission.impl.informationfinder;


import java.util.Calendar;

import org.apache.log4j.Logger;
import org.ggf.schemas.jsdl._2005._11.jsdl.JobDefinition_Type;
import org.globus.mds.glue.GLUECERPType;
import org.globus.mds.glue.StateType;

import se.umu.jobsubmission.util.JSDLUtils; 

import EDU.oswego.cs.dl.util.concurrent.ConcurrentReaderHashMap;

/**
 * @author tordsson
 *
 * This class is a cache for retrieved resource information. 
 * When information about a resource is required by the broker, 
 * it is first checked if a (recent enough) entry for that 
 * resource is found in the cache. When new information is 
 * retrieved during job submission, this is stored in the cache. 
 * This cache is coherent within the JSS, meaning that the load 
 * information for a resource is adjusted as jobs are submitted 
 * to that particular resource. 
 */
public class ResourceInformationCache {
	
	private static Logger logger = Logger.getLogger(
			ResourceInformationCache.class); 
	private ConcurrentReaderHashMap map; 
	
	/**
	 * 
	 */
	public ResourceInformationCache() {
		super(); 
		map = new ConcurrentReaderHashMap(); 
	}
	
	public CachedResourceInformation get(Object key) {
		return (CachedResourceInformation) map.get(key); 
	}
	
	public void put(Object key, GLUECERPType info, Calendar timestamp) {
		CachedResourceInformation newInfo = new CachedResourceInformation( 
				timestamp, info); 
		map.put(key, newInfo); 
	}
	
	// Recalculate number of jobs and free CPUs for this resource
	// after submitting a job
	public void update(GLUECERPType info, JobDefinition_Type jobDescription) {		
		
    	// Number of CPUs
    	int numberOfCPUs = JSDLUtils.getNumberOfCPUs(jobDescription);     	
    	
    	// Decrease number of free CPUs with the number of CPUs used by the job. 
    	StateType state = info.getComputingElement(0).getState(); 
    	int freeCPUs = state.getFreeCPUs(); 
    	// logger.warn("Resource had: " + freeCPUs + " free CPUs"); 
    	if((freeCPUs - numberOfCPUs) > 0)
    		state.setFreeCPUs(freeCPUs - numberOfCPUs); 
    	
    	//logger.warn("Resource now has: " + info.getComputingElement(0).
    	//		getState().getFreeCPUs() + " free CPUs"); 
    	
    	// Add an extra job. 
    	int numberOfJobs = state.getTotalJobs(); 
    	// logger.warn("Resource had " + numberOfJobs + " jobs"); 
    	state.setTotalJobs(numberOfJobs + 1); 
    	logger.debug("Resource now has " + info.getComputingElement(0).
    			getState().getTotalJobs() + " jobs"); 
	} 
	
}
