package org.nordugrid.job.mds;

import java.util.Hashtable;
import java.util.LinkedList;

import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;

public class ARCMDSDiscovery {
	
	private int timeout; 

    LinkedList GIISes = new LinkedList();

    LinkedList Clusters = new LinkedList();

    public ARCMDSDiscovery() {
    }

    public ARCMDSDiscovery(String u, int timeout) {
        if(u != null) 
        	GIISes.add(u);
        this.timeout = timeout;         
    }

    public void addGIIS(String u) {
        if(u == null) return;
        for(int i = 0;i<GIISes.size();i++) {
            if(((String)(GIISes.get(i))).equals(u)) return;
        }
        GIISes.add(u);
    }

    public void addCluster(String u) {
        if(u == null) return;
        for(int i = 0;i<Clusters.size();i++) {
            if(((String)(Clusters.get(i))).equals(u)) return;
        }
        Clusters.add(u);
    }

    public int haveGIISes() {
        return GIISes.size();
    }

    public int haveClusters() {
        return Clusters.size();
    }

    public String getGIIS(int i) {
        return (String)(GIISes.get(i));
    }

    public String getCluster(int i) {
        return (String)(Clusters.get(i));
    }

    public void DiscoverAll() throws ARCMDSDiscoveryError {
        // Go through all GIISes
        for(int i = 0;i<GIISes.size();i++) {
            String GIIS = (String)(GIISes.get(i));
            try {
                Hashtable env = new Hashtable();
                env.put(Context.INITIAL_CONTEXT_FACTORY,
                              "com.sun.jndi.ldap.LdapCtxFactory");
                
                env.put("com.sun.jndi.ldap.connect.timeout", ""+this.timeout);
                env.put(Context.PROVIDER_URL,GIIS);
                DirContext ctx = new InitialDirContext(env);
                String[] needed_attrs = { "giisregistrationstatus" };
                SearchControls controls = new SearchControls();
                controls.setCountLimit(0);            
                controls.setTimeLimit(this.timeout);
                controls.setSearchScope(SearchControls.OBJECT_SCOPE);
                controls.setReturningAttributes(needed_attrs);
                NamingEnumeration resultp = 
                              ctx.search("","(objectclass=*)",controls);
                // Each result represents registered GIIS or GRIS
                for(;resultp.hasMore();) {
                    SearchResult result = (SearchResult)(resultp.next());
                    Attributes attrs = result.getAttributes();
                    if(attrs.size() == 0) continue;
                    String url;
                    try {
                        // Reject nonVALID entries
                        String value = 
                           (String)(attrs.get("Mds-Reg-status").get());
                        if(!value.equals("VALID")) continue;
                        // Make URL
                        url = "ldap://";
                        url += (String)(attrs.get("Mds-Service-hn").get());
                        url += ":";
                        url += (String)(attrs.get("Mds-Service-port").get());
                        // Decide where to put it
                        String suffix = 
                           (String)(attrs.get("Mds-Service-Ldap-suffix").get());
                        // Strip spaces from parts of DN
                        int n = 0;
                        for(;;) {
                            n = suffix.indexOf(',',n);
                            if(n < 0) break;
                            suffix=suffix.substring(0,n).trim()+","+
                                   suffix.substring(n+1).trim();
                            n = suffix.lastIndexOf(',',n);
                            if(n < 0) break;
                            n++;
                        }
                        suffix.trim();
                        url+="/"+suffix;
                        if(suffix.substring(0,17).equalsIgnoreCase("Mds-Vo-name=local") || 
                        		suffix.substring(0,23).equalsIgnoreCase("nordugrid-cluster-name=")) {
                            // Cluster
                            addCluster(url);

                        } else if(suffix.substring(0,12).equalsIgnoreCase("Mds-Vo-name=")) {
                            // GIIS
                            addGIIS(url);
                        }

                    } catch(java.lang.Exception err) {
                        continue;
                    }
                }
            } catch (NamingException err) {
                continue;
            }
        }
    }

}

