package org.nordugrid.job.mds;

import java.util.List;
import java.util.LinkedList;
import javax.naming.directory.Attributes;
import javax.naming.directory.Attribute;

public class ARCMDSResource {

    private String mdsurl = null;
    private String joburl = null;
    private String name  = null;

    private int freejobs = -1;
    private int maxjobs  = -1;

    private String architecture = null;
    private String middleware = null;
    private String opsys = null;
    private List runtimeenvironment = new LinkedList();

    private int diskspace = 0;
    private int nodememory = 0;


    public ARCMDSResource(Attributes cluster,Attributes queue,Attributes user) {
        parseAttributes(cluster,queue,user);
    }

    private String setValue(String name,Attributes attrs,String value) {
        try {
            String v = (String)(attrs.get(name).get()); 
            if(v != null) value=v;
        } catch(java.lang.Exception err) {
        }
        return value;
    }

    private int setValue(String name,Attributes attrs,int value) {
        try {
            String v=setValue(name,attrs,null);
            Integer n = new Integer(v);
            value=n.intValue();
        } catch(java.lang.Exception err) {
        }
        return value;
    }

    private void setValues(String name,Attributes attrs,List values) {
        try {
            Attribute attr = attrs.get(name);
            for(int i=0;i<attr.size();i++) {
                String v = (String)(attrs.get(name).get(i));
                if(v == null) continue;
                int n = 0;
                for(;n<values.size();n++) {
                    if(v.equals((String)(values.get(n)))) break;
                }
                if(n >= values.size()) {
                    values.add(v);
                }
            }
        } catch(java.lang.Exception err) {
        }
    }

    private void parseAttributes(Attributes cluster,Attributes queue,Attributes user) {

        architecture=setValue("nordugrid-cluster-architecture",cluster,architecture);
        middleware=setValue("nordugrid-cluster-middleware",cluster,middleware);
        opsys=setValue("nordugrid-cluster-opsys",cluster,opsys);
        setValues("nordugrid-cluster-runtimeenvironment",cluster,runtimeenvironment);
        
        architecture=setValue("nordugrid-queue-architecture",cluster,architecture);
        middleware=setValue("nordugrid-queue-middleware",cluster,middleware);
        opsys=setValue("nordugrid-queue-opsys",cluster,opsys);
        setValues("nordugrid-queue-runtimeenvironment",cluster,runtimeenvironment);
        
        architecture=setValue("nordugrid-authuser-architecture",cluster,architecture);
        middleware=setValue("nordugrid-authuser-middleware",cluster,middleware);
        opsys=setValue("nordugrid-authuser-opsys",cluster,opsys);
        setValues("nordugrid-authuser-runtimeenvironment",cluster,runtimeenvironment);
        
        joburl=setValue("nordugrid-cluster-contactstring",cluster,joburl);
        // private String mdsurl;
        name=setValue("nordugrid-queue-name",queue,name);

        diskspace=setValue("nordugrid-cluster-sessiondir-free",cluster,diskspace);
        nodememory=setValue("nordugrid-queue-nodememory",queue,nodememory);

        int waitjobs = -1;

        int cpus = -1;
        int jobs = -1;
        int used = -1;
        int queued = -1;
        cpus=setValue("nordugrid-cluster-totalcpus", cluster,cpus);
//System.err.println("nordugrid-cluster-totalcpus: "+cpus);
        jobs=setValue("nordugrid-cluster-totaljobs", cluster,jobs);
//System.err.println("nordugrid-cluster-totaljobs: "+jobs);
        used=setValue("nordugrid-cluster-usedcpus",  cluster,used);
//System.err.println("nordugrid-cluster-usedcpus: "+used);
        queued=setValue("nordugrid-cluster-queuedjobs",cluster,queued);
//System.err.println("nordugrid-cluster-queuedjobs: "+queued);
        if((queued>=0) && (used>=0) && ((queued+used)>jobs)) jobs=queued+used;
        if((jobs>=0) && (cpus>=0)) {
            if(jobs > cpus) {
                freejobs=0;
                waitjobs=jobs-cpus;
            } else {
                freejobs=cpus-jobs;
                waitjobs=0;
            }
        }

        used = -1;
        int maxused = -1;
        queued = -1;
        int maxqueued = -1;
        used=setValue("nordugrid-queue-running",queue,used);
//System.err.println("nordugrid-queue-running: "+used);
        maxused=setValue("nordugrid-queue-maxrunning",queue,maxused);
//System.err.println("nordugrid-queue-maxrunning: "+maxused);
        queued=setValue("nordugrid-queue-queued", queue,queued);
//System.err.println("nordugrid-queue-queued: "+queued);
        maxqueued=setValue("nordugrid-queue-maxqueuable", queue,maxqueued);
//System.err.println("nordugrid-queue-maxqueued: "+maxqueued);
        if((cpus>=0) && (cpus<maxused)) maxused=cpus;
        cpus=-1;
        cpus=setValue("nordugrid-queue-totalcpus",queue,cpus);
        if((cpus>=0) && (cpus<maxused)) maxused=cpus;
        cpus=setValue("nordugrid-queue-maxuserrun",queue,cpus);
        if((cpus>=0) && (cpus<maxused)) maxused=cpus;
        if((used>=0) && (queued>=0)) used+=queued;
        if((maxused>=0) && (used>=0) && (maxused>used)) {
            if(((maxused-used) < freejobs) || (freejobs<0)) freejobs=maxused-used;
        }
        if((maxused>=0) && (maxqueued>=0)) {
            if(((maxused+maxqueued) < maxjobs) || (maxjobs<0)) maxjobs=(maxused+maxqueued);
        }
//System.err.println("freejobs: "+freejobs);
//System.err.println("maxjobs: "+maxjobs);

        int freecpu = -1;
        freecpu=setValue("nordugrid-authuser-freecpus",user,freecpu);
        if((freecpu>=0) && ((freecpu<freejobs) || (freejobs<0))) freejobs=freecpu;

        if(freecpu<0) freecpu=0;

        try {
        System.err.println("freejobs: "+freejobs);
        System.err.println("maxjobs: "+maxjobs);
        if(mdsurl!=null) System.err.println("mdsurl: "+mdsurl.toString());
        if(joburl!=null) System.err.println("joburl: "+joburl.toString());
        if(name!=null) System.err.println("name: "+name.toString());
        if(architecture!=null) System.err.println("architecture: "+architecture.toString());
        if(middleware!=null) System.err.println("middleware: "+middleware.toString());
        if(opsys!=null) System.err.println("opsys: "+opsys.toString());
        if(runtimeenvironment!=null) System.err.println("runtimeenvironment: "+runtimeenvironment.toString());
        System.err.println("diskspace: "+diskspace);
        System.err.println("nodememory: "+nodememory);
        } catch(java.lang.Exception err) {
            System.err.println("Error: "+err.toString());
        }

    }


}
