/* $Id: ngjm.cpp,v 1.9 2006/01/02 20:47:04 aleks Exp $ */
/* ngjm.cpp - see ngjm.h for futher information */
/* Copyright 2003 Henrik Thostrup Jensen and Jesper Ryge Leth
 * All rights reserved.
 *
 * This file is part of NGProxy.
 *
 * NGProxy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * NGProxy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with NGProxy; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
                                                                                          
/**
 * NGProxy will change name to "Nunabungo Gravdeha Jelahero Mongyput" /
 * NG Job Manager, or ngjm, but for now it still called NGProxy
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef _BSD_SOURCE
#define _BSD_SOURCE
#endif

#include <stdlib.h>
#include <errno.h>
#include <pthread.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>
#include <sys/wait.h>

#include <iostream>
#include <fstream>
#include <string>
#include <queue>
#include <list>

#include "Preferences.h"
#include "CertInfo.h"
#include "Xrsl.h"
#include "Giis.h"
#include "MdsQuery.h"
#include "JobSubmission.h"

#include "ngjm.h"
#include "ngjmlib.h"

#include "ngui.h"

/**
 * Interval for checking for updates (ca. in seconds)
 */
const int UPDATE_INTERVAL = 60; // Set to 300 when going real
                                                                                          
/**
  Default number of attempts to resubmit a job
 */
const int RESUBMIT_ATTEMPTS = 1;

/**
 * Wrapper function to start listener thread - dont call it
 * unless you know what you are doing
 */
static void* uds_listener(void *pointer);


/**
 * Main - should probably be replaced by call in main.cpp
 * so that it fits into the UI plugin structure
 */
int main(int argc, char** argv) {
	if (argc == 2) {
		setenv("MAXGRID", argv[1], true);
	}
	const bool dryrun = false;   // pretend mode
	const bool dumpxrsl = false; // dump the xrsl file send to server
	const int timeout = 4;      // timeout to servers - default = 40
	int s_timeout = 300;   // How long we want to wait for the job to change state
	char *stimeout = getenv("STIMEOUT");

	if (stimeout != 0) {
		s_timeout = atoi(stimeout);
	}

	const int debug = 0;         // debug level
	const bool anonymous = true; // anon when connecting to mds (i think)

	int result = ngproxyxx(dryrun, dumpxrsl, timeout, s_timeout, debug, anonymous);

	return result; // We either get 0 or 1, just return it
}


/**
 * Method to call from main
 * Creates a ngproxy object and starts it (created daemon)
 */
extern "C"
int ngproxyxx (const bool dryrun,
               const bool dumpxrsl,
               const int  timeout,
               const int  s_timeout,
               const int  debug,
               const bool anonymous) {

	// create proxy object
	NGProxy *ngproxy = new NGProxy (dryrun, dumpxrsl, timeout, s_timeout, debug, anonymous);

	int retval = ngproxy->Start(false);
	// Delete the object (we've either fork()ed or failed)
	delete ngproxy;
	if (retval != 0) {
		cerr << "Error starting ngproxy" << endl;
		return 1;
	}

	return 0;
}


NGProxy::NGProxy(const bool dryrun,
                 const bool dumpxrsl,
                 const int  timeout,
		 const int  s_timeout,
                 const int  debug,
                 const bool anonymous) {

	// save the arguments
	this->dryrun    = dryrun;
	this->dumpxrsl  = dumpxrsl;
	this->timeout   = timeout;
	this->s_timeout = s_timeout;
	this->debug     = debug;
	this->anonymous = anonymous;
	this->xrslsocket = -1;
	this->num_current_jobs = 0;

}

NGProxy::~NGProxy() {

	// Various clean up goes here
	if (logfile) {
		logfile.close();
	}
	if (ngproxy_socket != "") {
		unlink(ngproxy_socket.c_str());
	}

}


int NGProxy::Start(bool create_daemon = true) {
	// Maximum number of jobs in grid is stored in the variable
	// MAXGRID
	char *maxgrid = getenv("MAXGRID");
	if (maxgrid)
	    this->maxgridjobs = atoi(maxgrid);
	else
	    this->maxgridjobs = 30;

	// create ngproxy home directory and logfile stream
	ngproxy_home = getenv("HOME");
	if (ngproxy_home.empty()) {
		cerr << "Error getting envrioment for home path" << endl;
		return -1;
	}
	ngproxy_home.append("/.ngproxy/");

	// Try to create directory (with read,write,execute for user)
	int retval = mkdir(ngproxy_home.c_str(), (mode_t)S_IRWXU);
	if ((retval != 0) && (errno != EEXIST)) {
		// Fail on all errors except previous existence
		cerr << "Error creating proxy home at " << ngproxy_home << endl;
		return -2;
	}

	// Open log file - start by creating logfile pathname
	string logfile_name = ngproxy_home + "ngproxy.log";
	// Open the file - append and write mode
	logfile.open(logfile_name.c_str(), ios::app|ios::out); 
	if (!logfile) {
		cerr << "Failed to open logfile: " << logfile_name << endl;
		return -3;
	}

	// Set socket path - needed when creating unix domain soclet	
	ngproxy_socket = ngproxy_home + "xrsl";

	// Lets get the globus modules up and running
	ActivateGlobus();

	// See if we have user proxy
	//user_proxy = new CertInfo();
	if (!user_proxy) { // Defined in header file
		// Failed to get user proxy
		cerr << "Error getting user proxy" << endl;
		return -4;
	}

	// Sanity check over - now we start creating the daemon
	int pid; // Defined here since c++ does not allow goto's over definitions

	// If create_deamon is set to false we skip all the daemonization
	// stuff. This can be usefull for debugging and such
	if (!create_daemon) {
		cout << "Skipping daemon creation" << endl;
		goto further_down;
	}

	cout << "Creating daemon" << endl;

	// Begin doing standard daemon creation
	// Start by creating new process
	pid = fork();
	switch(pid) {
		case -1: // Error
			cerr << "Error during fork() when creating daemon" << endl;
			return -5;
		case 0: // Child
			break;
		default: // Parent
			return 0; // From now the child is on its own
	}

	/* We are now in child process
	 * Now we create a new session, so we dont exist when
	 * our parent exit.
	 */
	pid = setsid();
	if (pid == -1) {
		cerr << "Error creating new process group" << endl;
		exit(1); // Remeber that we are in child
	}

	/* Even though we have detached us from the parent session group
	 * we can still get access to a terminal (and we dont want that),
	 * since we are (new) session leader. So we fork() again
	 */
	pid = fork();
	switch(pid) {
		case -1: // Error
			cerr << "Error during fork() when daemonizing" << endl;
		case 0: // Child
			break;
		default: // Parent
			exit(0); // Dont think this will be read by anything
	}

	/* Now we are in the second child and our parent has exited
	 * This means that we are a daemon.
	 * Also we are not a process leader meaning that we cannot
	 * claim a terminal (good).
	 *
	 * Before we start doing anything we do some standard stuff
	 * We start by closing our file descriptors
	 */
	close(0); // close std in
	close(1); // close std out
	close(2); // close std err

	/* Create new filedescriptors - this should be done
	 * even there is no intention of using them
	 *
	 ' We know that we get the lowest possible file desciptors
	 * because posix says that is must be so
	 */
	open("/dev/null",O_RDONLY); // std in
	open("/dev/null",O_WRONLY); // std out
	open("/dev/null",O_WRONLY); // std err

further_down:

	chdir("/"); // Avoid locking a mount point
	umask((mode_t)~(S_IRUSR | S_IWUSR)); // set umask ~600

	MainLoop(); // We never return from this

	return 0; // Make gcc shut up
}



void NGProxy::Log(string msg1, string msg2 = "", string msg3 = "") {

	// Get current time so we can put it in the log
	time_t current_time = time(0);
        char my_time_buf[26];
	// Create a string from current_time
	string my_time = string(ctime_r(&current_time, my_time_buf));
	// Do stupid stuff to remove newline from ctime returned string
	string my_time_chopped = my_time.substr(0,my_time.size()-1);
	// Write timestamp and message to log
	logfile << my_time_chopped << ": " << msg1 << msg2 << msg3 << endl;
}


void NGProxy::MainLoop() {

	// Standard posix thread init
	pthread_attr_t thread_attr;
	pthread_attr_init(&thread_attr);
	pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_JOINABLE);

	// create thread for listening on unix domain socket
	pthread_t listener_thread;
	int retval = pthread_create(&listener_thread, 0, uds_listener, this);
	if (retval != 0) {
		Log("Error creating UDS Listener thread, exiting");
		exit(1);
	}

	// Init mutex variable and other stuff used in mainloop
	pthread_mutex_init(&new_jobs_lock, NULL);
	pthread_mutex_init(&current_jobs_lock, NULL);
	unsigned int time_left = 0;
	unsigned int last_time = time(0);
	bool running = true;

	// We have liftoff
	Log("=====  NG Proxy started  =====");
	/**
	 * ===================================
	 *
	 * Main loop start
	 *
	 * ===================================
	 */
	do {

		// Check if it time tu update job status
		if (time(0) - last_time > UPDATE_INTERVAL) {
			UpdateJobStatus();
			last_time = time(0);
		}

		HandleNewJobs();
		
		time_left = sleep(1);
		if (time_left) {
			// If we get here, we where interrupted during sleep
			Log("The sleeper awakens"); // We should do clean up here
		}

                // Clean the forked downloads
                int size = forked_downloads.size();
                list<pid_t>::iterator cpid = forked_downloads.begin();

                while (cpid != forked_downloads.end()) {
                    int rval = waitpid(*cpid, NULL, WNOHANG); // try to clean
                    if (rval > 0) {
                        forked_downloads.erase(cpid++);
                    }
                    else {
                        cpid++;
                    }
                }

	} while (running);
	/**
	 * ===================================
	 * 
	 * Main loop stop
	 *
	 * ===================================
	 */

	Log("Exiting main loop");
}


void* uds_listener(void* pointer) {

	// Do stupid typecast (thread functions must use void pointers)
	NGProxy* ngproxy = (NGProxy*) pointer;
	// Start listening
	int retval = ngproxy->Listen();
	// Exit thread (we only get here on failure)
	pthread_exit((void *) retval);
}


int NGProxy::Listen() {

	// Create socket
	int server_socket = socket(AF_UNIX, SOCK_STREAM,0);
	if (server_socket == -1) {
		Log("Error creating socket");
		return -1;
	}

	// Set linger (timeout) close connection options
	{
		int on = 1;
		setsockopt(server_socket,SOL_SOCKET,SO_REUSEADDR,(void*)(&on),sizeof(on));
	};	

	// Create socket address struct used for binding
	struct sockaddr_un server;
	server.sun_family = AF_UNIX;
	strcpy(server.sun_path, ngproxy_socket.c_str());

	// Delete any previous file
	unlink(ngproxy_socket.c_str());

	int retval; // used for return values

	// Bind socket to file
	retval = bind(server_socket, (struct sockaddr*) &server, sizeof(server));
	if (retval == -1) {
		Log("Error binding socket to file ", ngproxy_socket);
		return -1;
	}

	// Start listening
	retval = listen(server_socket,2);
	if (retval == -1) {
		Log("Listen failed");
		return -1;
	}
	Log("Listener thread started");

	struct sockaddr_un client;
	int addrlen = sizeof(client);

	// Start accepting connections
	for (;;) {
		// Wait for connection
		int socket_handle = accept(server_socket, (struct sockaddr*)&client,
		                           (socklen_t*) &addrlen);
		if (socket_handle == -1) {
			Log("Error accepting connetion");
		}
		else {
			Log("Accepted Connection");
			// If we want to respond faster to multiple connections we 
			// could create a thread here
			xrslsocket = socket_handle;
			HandleConnection(socket_handle);
		}
		usleep(50000); // .05 seconds (creates a max of 20 connections pr sec)
	}
	return 0;
}


void NGProxy::HandleConnection(int socket_handle) {

	const int MAX_BUFFER = 1024;
	char buffer[MAX_BUFFER];
	// EOF needs to be cleared so we need a sockstream because of
	// the interface of clearerr.
	FILE *sockstream = fdopen(socket_handle, "rw");

	bool ok = 1;

	while (ok) {
		int bytes_read = 0;
		int read_still = 1;
		string recieved_message = "";
		/* Read everything from the socket, then create an
		 * xrsl object from the string recived and attempt
		 * to submit it.
		 *
		 * You could do nasty stuff if you just kept sending stuff
		 * to it, but we dont care about that right now
		 */
		do {
			bytes_read = read(socket_handle,buffer,MAX_BUFFER);
			switch (bytes_read) {
				case -1: // Error
					Log("Error during read from socket");
					ok = 0;
					break;
				case 0: // Read EOF (done reading)
					Log("Read EOF");
					clearerr(sockstream);
					read_still = 0;
					ok = 0;
					goto lost_connection;
					break;
				default: { // Reading
					string partial_message (buffer, bytes_read);
					if (partial_message[partial_message.size()-1] == EOF) {
						Log("Received EOF");
						read_still = 0;
						clearerr(sockstream); // Get on with reading
						partial_message[partial_message.size()-1] = '\0';
					}

					recieved_message += partial_message;
					// FIXME we need size check here
				}
			}

		} while (read_still);

		// Ok now were done reading
		// Start by closing the socket handle
//		int retval = close(socket_handle);
//		if (retval == -1) {
//			Log("Error closing socket handle (weird)");
//		}

		// Create xrsl object
		Xrsl* xrsl = new Xrsl(recieved_message);
		if (!(*xrsl)) {
			Log("Recieved broken xrsl, will not submit");
			return;
		}

		
		// Grap xrsl vector lock, push the Xrsl object into
		// it, and realease lock
		pthread_mutex_lock(&new_jobs_lock);
		new_jobs.push(xrsl);
		pthread_mutex_unlock(&new_jobs_lock);
		recieved_message = "";

		Log("New job was put into queue");
	}
	return;
lost_connection:
	Log("Connection was lost");
	return;
}


void NGProxy::HandleNewJobs() {
	/**
	 * We now check for the arrival of new jobs
	 * This is done by acquiring lock, see if there is anything
	 * in the new_jobs vector, we take one out, release the lock.
	 * and then we submit the job.
         * If there are no new jobs and we get here, it means the queue
         * for sending jobs is empty, so we might want to ask more jobs.
         * Call the procedure...
	 */
	bool new_job = false;    // Indicates whethere there are any new jobs
	Xrsl* xrsl = 0;
	pthread_mutex_lock(&new_jobs_lock);

	if (!new_jobs.empty()) {  // See if any new jobs arrived
		Log("New job(s) appeared in queue");
		// Take the first element in the queue and pop it
		xrsl = new_jobs.front();
		new_jobs.pop();
		new_job = true;
	}
	pthread_mutex_unlock(&new_jobs_lock); // Release lock

	if (new_job) {
		string jobid = SubmitJob(xrsl, "");
                printf("%s\n", jobid.c_str());
		if (jobid.empty()) {
			// Submission failed - do nothing
			// Hey? Are we leaking here sometimes?
			Log("Job submission failed - mem leak");
			return;
		}
		else {
			// Job was submitted succesfully
			// Create jobstatus object and push onto current_jobs
			JobStatus* js = new JobStatus(xrsl, jobid);
			current_jobs.push_back(js); 
			// We need a counter telling how many jobs are
			// in current jobs
			pthread_mutex_lock(&current_jobs_lock);
			num_current_jobs += 1;
			pthread_mutex_unlock(&current_jobs_lock);
		}
	} else {
		GetMoreJobs();
	}

	return;
}

void NGProxy::GetMoreJobs() {
	if (xrslsocket != -1) {
		pthread_mutex_lock(&current_jobs_lock);
		cout << "Number of jobs in grid: " << num_current_jobs << endl;
		if (num_current_jobs < this->maxgridjobs) {
			Log("Asking for more jobs from the socket");
			write(xrslsocket, "# getjob 1\n", 11);
		} else {
			Log("Grid is full");
		}
		pthread_mutex_unlock(&current_jobs_lock);
	}
	return;
}

string NGProxy::SubmitJob(Xrsl* xrsl, string old_cluster = "") {

	// Check if we got a proper xrsl object
	if (!(*xrsl)) {
		Log("Attempt to submit bad/broked xrsl, will not submit");
		delete xrsl;
		return "";
	}

	// init and check xrsl description
	xrsl->Eval();
	if (xrsl->Check()) {
		Log("Error checking xrsl description");
		delete xrsl;
		return "";
	}

	// We need these later
        vector <Xrsl> xrsl_list;
	vector <Cluster> cluster_list;
	vector <Giis> giis_list;

	// Put xrsl object in a vector (dont think - just do)
	xrsl_list.push_back(*xrsl);

	// Lets get some clusters
	// We start be building a giis list
	vector<string> esv; // empty string vector
	if (GetGiises(esv,giis_list)) {
		// error
		Log("Error getting giis list during job submission");
		delete xrsl;
		return "";
	}

	// Find clusters from giis info
	cluster_list = FindClusters(giis_list, user_proxy.GetSN(),
	                            anonymous, timeout , debug);

	if (cluster_list.empty()) {
		Log("Could not find any clusters for job submission");
		delete xrsl;
		return "";
	}

	// Get cluster info about the clusters found
	FindClusterInfo(cluster_list, Mds::JobSubmission, user_proxy.GetSN(),
	                anonymous, timeout, debug);

	string joblist;    // Empty (unknown use)

	// Attempt to submit the job
	string jobid = "";
	int retval = JobSubmission(cluster_list, old_cluster, *xrsl, joblist,
	                           dryrun, dumpxrsl, timeout, debug, &jobid);

	if (retval) {
		Log("Error during job submission");
		delete xrsl;
		return "";
	}

	if (debug) {
		cout << "Jobid: " << jobid << endl;
	}
	// Write to log and push new jobstatus into current_jobs list
	Log("Job submitted with id ", jobid);
	// XXX Write instrumentation here

	return jobid;
}


void NGProxy::UpdateJobStatus() {

	/**
	 * Iterate over the jobs, get the status for each one,
	 * and act if the state is "FINISHED" or "FAILED"
	 */

	list<JobStatus*>::iterator jsi = current_jobs.begin(); 
	bool incremented = false;
	
	while (jsi != current_jobs.end()) {
	
		incremented = false;
		if (debug) {
			cout << "Pulling info for job " << (*jsi)->getJobid() << endl;
		}
		// Construct job object, info will be retrievd fom MDS
		// Get the stuff we want
		Job* job = getJob((*jsi)->getJobid());
		string status = "";
		string error = "";
		
		if (job == NULL) {
			Log("Job " + (*jsi)->getJobid() + " is sent but can't be seen (yet)");
			status = "NOTFOUND";
			error = "";
		}
		else {
		    status = job->GetStatus();
		    error  = job->GetErrors();
		}
		string prev_status  = (*jsi)->getPrevStatus();
		time_t prev_time    = (*jsi)->getPrevTime();
		time_t current_time = time(NULL);
		(*jsi)->setNewStatus(status);


		if (debug) {
			cout << "Status change " << (*jsi)->getJobid() << endl;
			cout << "  " << prev_status << " => " << status;
                        cout << " (" << current_time - prev_time << ")" << endl;
		}

		if (strncmp(status.c_str(),"FINISHED", 8) == 0) {

			// Check if error occured during job execution
			if (error.empty()) {
				// Normal job exit
				Log("Job ", (*jsi)->getJobid(), 
					" finished correctly and was removed from current job list");
				if (debug) {
					cout << "Job " << (*jsi)->getJobid() << " finished - removing" << endl;
				}
				// Fetch the job (safe, as not found
				// jobs are not in state FINISHED)
				FetchJob((*jsi)->getJobid().c_str());
                                // Free memory of jsi
                                JobStatus* ojsi = *jsi;
				current_jobs.erase(jsi++);
                                delete ojsi;
				pthread_mutex_lock(&current_jobs_lock);
				-- num_current_jobs;
				pthread_mutex_unlock(&current_jobs_lock);
				incremented = true;
				// XXX Write instrumentation here:
				// finished correctly
			}
			// Error occured during job execution
			else {
				if (debug) {
					cout << "Job with id " << (*jsi)->getJobid() << " failed" << endl;
				}
				// handle job failure here
				bool job_resubmitted = HandleJobFailure(*jsi);
				if (!job_resubmitted) {
					// If the job wasn't resubmitted we remove it
                                        JobStatus* ojsi = *jsi;
					// XXX write instrumentation
					// here:
					// failure + deleted
					current_jobs.erase(jsi++);
                                        delete ojsi;
					pthread_mutex_lock(&current_jobs_lock);
					-- num_current_jobs;
					pthread_mutex_unlock(&current_jobs_lock);
					incremented = true;
				}
				else {
					// XXX write instrumentation
					// here:
					// failure
                                }
			}
		}

		else if ((strncmp(status.c_str(), "INLRMS", 6) == 0) &&
			 (strncmp(status.c_str(), "INLRMS: Q", 9) != 0)) {
			// Job is running
		}

		// Check that the job has been doing some progress in
		// grid.
		// The longest status string is MAXSTATUSLENGTH chars
		else if (strncmp(status.c_str(), prev_status.c_str(), MAXSTATUSLENGTH) == 0) {
			if (debug) {
				cout << "Job with id " << (*jsi)->getJobid() << " has not changed" << endl;
			}
			if (current_time - prev_time > s_timeout) {
				// Job has been on the same state for
				// too long.
				bool job_resubmitted = HandleJobFailure(*jsi);
				if (!job_resubmitted) {
					Log("Job ", (*jsi)->getJobid(), " retry limit reached");
                                        JobStatus *ojsi = *jsi;
					current_jobs.erase(jsi++);
					// XXX Write instrumentation here:
					// state stall + deleted
                                        delete ojsi;
					pthread_mutex_lock(&current_jobs_lock);
					-- num_current_jobs;
					pthread_mutex_unlock(&current_jobs_lock);
					incremented = true;
				}
				// XXX Write instrumentation here
				// state stall
			}
		}

		if (!incremented) {
			jsi++;
		}
//		else if (status.compare("FAILED") == 0) {
//			cout << "Job with id " << (*jsi)->getJobid() << "failed" << endl;
//			// handle job failure here
//			HandleJobFailure(*jsi);
//		}
//		// Delete the job
		delete job;
	}

	return;
}



bool NGProxy::HandleJobFailure(JobStatus* js) { 

	string jobid = js->getJobid();

	cout << "Job " << jobid << " failed - attempting to recover" << endl;

	if (js->getAttempts() >= RESUBMIT_ATTEMPTS) {
		// If the job has been submitted more than three times
		// and still failed we throw it out
		if (debug) {
			cerr << "Job " << jobid << " has been re-submitted "
			     << RESUBMIT_ATTEMPTS << " times - removing it from job list";
		}
		Log("Job ", jobid, " was removed from current job list, due to maximum number of submissions attempts being reached");
		return false; // Job didnt get resubmitted 
	}
	// If we get here we attempt to resubmit the job

	Xrsl* xrsl = js->getXrsl();
	string old_cluster = getClusterName(js->getJobid());

	string new_jobid = SubmitJob(xrsl, old_cluster);

	js->newSubmission(new_jobid);
	Log(jobid, " was resubmitted as ", new_jobid);

        js->setNewStatus("RESUB");
	
	return true; // jobs was sumbitted again
}


void
NGProxy::FetchJob(const char *jobid) {

//	cout << "Fetching job " << job->GetName() << " as soon as it's implemented" << endl;

        pid_t pid = fork();
        if (pid == 0) {
            execlp("ngget", "ngget", jobid, (char *)NULL);
        }
        // In parent
        forked_downloads.push_back(pid);
	return;
}

// ========================================================
// JobStatus class from here
// ========================================================


JobStatus::JobStatus(Xrsl* xrsl, string jobid) {

	this->attempts = 0;          // Init number of attempts
	this->xrsl = xrsl;           // Save xrsl
	this->current_jobid = jobid; // Save jobid
	this->prev_status = "INITIAL";
	this->prev_time = time(NULL);

	// Put the cluster name into the cluster vector
	string current_cluster = getClusterName(jobid);
	this->clusters.push_back(current_cluster);

}


JobStatus::~JobStatus() {

	delete xrsl;

	return;
}


Xrsl* JobStatus::getXrsl() {
		
	return xrsl;
}


string JobStatus::getJobid() {

	return current_jobid;
}


int JobStatus::getAttempts() {

	return attempts;
}

string JobStatus::getPrevStatus() {

	return prev_status;
}

time_t JobStatus::getPrevTime() {

	return prev_time;
}

void JobStatus::setNewStatus(string s) {

	// We only want to update the time if the state has changed
	if (strncmp(prev_status.c_str(), s.c_str(), MAXSTATUSLENGTH) != 0) {
		prev_time = time(NULL);
		prev_status = s;
	}
	return;
}


void JobStatus::newSubmission(string new_jobid) {

	clusters.push_back(getClusterName(current_jobid));
	current_jobid = new_jobid;
	attempts++;

}
